<?php
if ( ! class_exists( 'IPTrank_Engine' )) {

    require_once( IPTRANK_PATH . "includes/simple_html_dom.php" );
    
    /**
     * Class engine for get rank search engine
     */
    class IPTrank_Engine
    {
        /**
         * Take keyword
         * @var string
         */
        protected $_keyword;
        
        /**
         * Take site
         * @var string
         */
        protected $_site;
        
        /**
         * Is first time process
         * @var boolean
         */
        protected $_first_time;
        
        /**
         * Take search result
         * @var string
         */
        protected $_search_result;
        
        /**
         * Take search urls result
         * @var array
         */
        protected $_search_urls;
        
        /**
         * Take search error
         * @var string
         */
        protected $_search_error;
        
        /**
         * Take rank results
         * @var string
         */
        protected $_rank_result;
        
        /**
         * Take rank
         * @var int
         */
        protected $_rank = 0;
        
        /**
         * Class constructor
         *
         * @param string $keyword
         * @param string $site
         * @return void
         */
        public function __construct( $keyword, $site )
        {
            $this->set_keyword( $keyword )
                 ->set_site( $site );
        }
        
        /**
         * Get keyword
         *
         * @return string
         */
        public function get_keyword()
        {
            return $this->_keyword;
        }
        
        /**
         * Get site
         *
         * @return string
         */
        public function get_site()
        {
            return $this->_site;
        }
        
        /**
         * Get rank
         *
         * @return int
         */
        public function get_rank()
        {
            return $this->_rank;
        }
        
        /**
         * Get rank result
         *
         * @return array
         */
        public function get_rank_result()
        {
            return $this->_rank_result;
        }
        
        /**
         * Get search result
         *
         * @return array
         */
        public function get_search_result()
        {
            return $this->_search_result;
        }
        
        /**
         * Get search error
         *
         * @return array
         */
        public function get_search_error()
        {
            return $this->_search_error;
        }
        
        /**
         * Get search urls
         *
         * @return array
         */
        public function get_search_urls()
        {
            return $this->_search_urls;
        }
        
        /**
         * Set keyword
         *
         * @param  string $keyword
         * @return object
         */
        public function set_keyword( $keyword )
        {
            $this->_keyword = apply_filters( 'pre_term_name', $keyword );
            return $this;
        }
        
        /**
         * Set site
         *
         * @param  string $site
         * @return object
         */
        public function set_site( $site )
        {
            $this->_site = sanitize_url( $site );
            return $this->_site;
        }
        
        /**
         * Get search engine host url
         *
         * @return string
         */
        public function get_search_engine_host()
        {
            $default = 'http://www.google.com/';
            $url     = get_option( 'iptrank_google_country' );
            if ( ! empty( $url )) {
                $url = urldecode( $url );
            } else {
                $url = $default;
            }
            return $url;
        }
        
        /**
         * Get search result data
         *
         * @return array
         */
        public function get_result_data()
        {
            $data = array(
                'keyword'   => $this->get_keyword(),
                'site'      => $this->get_site(),
                'error'     => $this->get_search_error(),
                'urls'      => $this->get_search_urls(),
                'ranks'     => $this->get_rank_result()
            );
            $data = maybe_serialize( $data );
            return $data;
        }
        
        /**
         * Run
         *
         * @return int
         */
        public function run()
        {
            $keyword = $this->get_keyword();
            $url     = $this->get_site();
            $se_host = $this->get_search_engine_host();
            $se_url  = $se_host . 'search?q='.$keyword.'&num=100&pws=0';
            
            //Get user agent randomly
            $user_agents  = self::get_user_agents();
            $user_agent   = $user_agents[rand(0,count($user_agents)-1)];
            
            $urls = array();
            $html = $this->_get_search_content( $user_agent, $se_url );
            if ( ! empty( $html )) {
                $urls = $this->_get_search_urls( $html );
            }
            $this->_search_result = $html;
            $this->_search_urls   = $urls;
            
            $rank = 0;
            if ( ! empty( $urls )) {
                $ranks = $this->_get_rank_url( $url, $urls );
                if ( ! empty( $ranks )) {
                    $rank = $this->_get_target_rank( $url, $ranks );
                }
            }
            return $this->_rank = $rank;
        }
        
        /**
         * Do request to search engine
         *
         * @param  string $user_agent
         * @param  string $url
         * @return string
         */
        protected function _get_search_content( $user_agent, $url )
        {
            $opts = array(
                'http' => array(
                    'user_agent' => $user_agent
                )
            );
            $context = stream_context_create( $opts );
            $html    = @file_get_html( $url, false, $context );
            return $html;
        }
        
        /**
         * Do request to search engine
         *
         * @param  object $html
         * @return string
         */
        protected function _get_search_urls( $html )
        {            
            $urls = array();
            $i    = 0;
            foreach($html->find('li[class=g]') as $element) {
                foreach($element->find('h3[class=r]') as $h3) {
                    foreach($h3->find('a') as $a) 
                    {
                        $urls[$i] = $a->href;
                    }
                }
                $i++;
            }
            return $urls;
        }
        
        /**
         * Do request to search engine
         *
         * @param  string $target
         * @param  array $urls
         * @return int
         */
        protected function _get_rank_url( $target, $urls )
        {
            $rank_result = array();
            
            $rank = 0;
            foreach( (array)$urls as $url )
            {
                $rank++;
                $url = $this->_fix_gsearch_url( $url );
                if ( $this->_check_host( $target, $url )) {
                    $rank_result[$rank] = $url;
                }
            }
            $this->_rank_result = $rank_result;
            return $rank_result;
        }
        
        /**
         * Check host
         *
         * @param  string $target
         * @param  string $url
         * @return boolean
         */
        protected function _check_host( $target, $url )
        {
            $target_host = $this->_get_url_host( $target );
            $url_host    = $this->_get_url_host( $url );
            if ( $target_host == $url_host ) {
                return true;
            }
            return false;
        }
        
        /**
         * Check url
         *
         * @param  string $target
         * @param  string $url
         * @return boolean
         */
        protected function _check_url( $target, $url )
        {
            $target = trim( $target, '/' );
            $url    = trim( $url, '/' );
            
            $target = $this->_get_url_no_www( $target );
            $url    = $this->_get_url_no_www( $url );
            
            if ( $target == $url ) {
                return true;
            }
            return false;
        }
        
        /**
         * Do request to search engine
         *
         * @param  string $url
         * @return int
         */
        protected function _get_url_host( $url )
        {
            $result  = 'http://';
            $parsed  = parse_url( $url );
            $host    = $this->_trim_www( $parsed['host'] );
            $result .= $host;
            return $result;
        }
        
        /**
         * Trim www
         *
         * @param  string $host
         * @return int
         */
        protected function _trim_www( $host )
        {
            if ( strpos( $host, 'www' ) !== false ) {
               $host = str_replace( 'www.', '', $host ); 
            }
            return $host;
        }
        
        /**
         * Get url without www
         *
         * @param  string $url
         * @return string
         */
        protected function _get_url_no_www( $url )
        {
            $parsed  = parse_url( $url );
            $result  = 'http://';
            if ( ! empty( $parsed['host'] )) {
                $result .= $this->_trim_www( $parsed['host'] );
            }
            if ( ! empty( $parsed['path'] )) {
                $result .= $parsed['path'];
            }
            return $result;
        }
        
        /**
         * Fix google search result url
         *
         * @param  string $url
         * @return string
         */
        protected function _fix_gsearch_url( $url )
        {
            $fixed  = $url;
            $parsed = parse_url( $url );
            if ( ! empty( $parsed['query'] )) {
                parse_str( $parsed['query'], $args );
                if ( $args && ! empty( $args['q'] )) {
                    $fixed = $args['q'];
                }
            }
            return $fixed;
        }
        
        /**
         * Get target rank
         *
         * @param  string $target
         * @param  array $ranks
         * @return int
         */
        protected function _get_target_rank( $target, $ranks )
        {
            $rank = 101;
            if ( ! empty( $ranks )) {
                foreach( $ranks as $idx => $url ) {
                    if ( $this->_check_url( $target, $url )) {
                        $rank = $idx;
                        break;
                    }
                }
            }
            return $rank;
        }
        
        /**
         * Get user agent
         *
         * @return int
         */
        static public function get_user_agents()
        {
            $user_agents = array(
                'Mozilla/5.0 (Windows; U; Windows NT 6.1; pl; rv:1.9.2.3) Gecko/20100401 Firefox/3.6.3',
                'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322)',
                'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30)',
                'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)',
                'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; en-GB)',
                'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; MS-RTC LM 8)',
                '(Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0))',
                'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.0',
                'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 8.50',
                'Opera/9.20 (Windows NT 6.0; U; en)',
                'Opera/9.30 (Nintendo Wii; U; ; 2047-7;en)',
                'Opera 9.4 (Windows NT 6.1; U; en)',
                'Opera/9.99 (Windows NT 5.1; U; pl) Presto/9.9.9',
                'Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.2 (KHTML, like Gecko) Chrome/6.0',
                'Mozilla/5.0 (Macintosh; U; Intel Mac OS X; de-de) AppleWebKit/522.11.1 (KHTML, like Gecko) Version/3.0.3 Safari/522.12.1',
                'Mozilla/5.0 (Windows; U; Windows NT 5.1; fr-FR) AppleWebKit/523.15 (KHTML, like Gecko) Version/3.0 Safari/523.15',
                'Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/523.15 (KHTML, like Gecko) Version/3.0 Safari/523.15',
                'Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_2; en-gb) AppleWebKit/526+ (KHTML, like Gecko) Version/3.1 iPhone',
                'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; en-us) AppleWebKit/525.25 (KHTML, like Gecko) Version/3.2 Safari/525.25',
                'Mozilla/5.0 (Windows; U; Windows NT 6.0; ru-RU) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16',
                'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_7; en-us) AppleWebKit/533.4 (KHTML, like Gecko) Version/4.1 Safari/533.4',
                'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:11.0) Gecko Firefox/11.0', 
                'Mozilla/5.0 (Windows; U; MSIE 9.0; WIndows NT 9.0; en-US))',
                'Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)',
                'Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; InfoPath.1; SV1; .NET CLR 3.8.36217; WOW64; en-US)',
                'Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11',
                'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.66 Safari/535.11',
                'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.24 (KHTML, like Gecko) Chrome/19.0.1055.1 Safari/535.24'
            );
            return $user_agents;
        }
        
        /**
         * Get google urls
         *
         * @return int
         */
        static public function get_google_urls()
        {
            $urls = array(
                'http://www.google.com/'        =>
                    __('Default', 'iptrank') . ' Google.com (http://www.google.com/)',
                'http://www.google.as/'         =>
                    __('American Samoa', 'iptrank') . ' (http://www.google.as/)',
                'http://www.google.off.ai/'     =>
                    __('Anguilla', 'iptrank') . ' (http://www.google.off.ai/)',
                'http://www.google.com.ag/'     =>
                    __('Antigua and Barbuda', 'iptrank') . ' (http://www.google.com.ag/)',
                'http://www.google.com.ar/'     =>
                    __('Argentina', 'iptrank') . ' (http://www.google.com.ar/)',
                'http://www.google.com.au/'     =>
                    __('Australia', 'iptrank') . ' (http://www.google.com.au/)',
                'http://www.google.at/'         =>
                    __('Austria', 'iptrank') . ' (http://www.google.at/)',
                'http://www.google.az/'         =>
                    __('Azerbaijan', 'iptrank') . ' (http://www.google.az/)',
                'http://www.google.be/'         =>
                    __('Belgium', 'iptrank') . ' (http://www.google.be/)',
                'http://www.google.com.br/'     =>
                    __('Brazil', 'iptrank') . ' (http://www.google.com.br/)',
                'http://www.google.vg/'         =>
                    __('British Virgin Islands', 'iptrank') . ' (http://www.google.vg/)',
                'http://www.google.bi/'         =>
                    __('Burundi', 'iptrank') . ' (http://www.google.bi/)',
                'http://www.google.ca/'         =>
                    __('Canada', 'iptrank') . ' (http://www.google.ca/)',
                'http://www.google.td/'         =>
                    __('Chad', 'iptrank') . ' (http://www.google.td/)',
                'http://www.google.cl/'         =>
                    __('Chile', 'iptrank') . ' (http://www.google.cl/)',
                'http://www.google.com.co/'     =>
                    __('Colombia', 'iptrank') . ' (http://www.google.com.co/)',
                'http://www.google.co.cr/'      =>
                    __('Costa Rica', 'iptrank') . ' (http://www.google.co.cr/)',
                'http://www.google.ci/'         =>
                    __('Côte d\'Ivoire', 'iptrank') . ' (http://www.google.ci/)',
                'http://www.google.com.cu/'     =>
                    __('Cuba', 'iptrank') . ' (http://www.google.com.cu/)',
                'http://www.google.cd/'         =>
                    __('Dem. Rep. of the Congo', 'iptrank') . ' (http://www.google.cd/)',
                'http://www.google.dk/'         =>
                    __('Denmark', 'iptrank') . ' (http://www.google.dk/)',
                'http://www.google.dj/'         =>
                    __('Djibouti', 'iptrank') . ' (http://www.google.dj/)',
                'http://www.google.com.do/'     =>
                    __('Dominican Republic', 'iptrank') . ' (http://www.google.com.do/)',
                'http://www.google.com.ec/'     =>
                    __('Ecuador', 'iptrank') . ' (http://www.google.com.ec/)',
                'http://www.google.com.sv/'     =>
                    __('El Salvador', 'iptrank') . ' (http://www.google.com.sv/)',
                'http://www.google.fm/'         =>
                    __('Federated States of Micronesia', 'iptrank') . ' (http://www.google.fm/)',
                'http://www.google.com.fj/'     =>
                    __('Fiji', 'iptrank') . ' (http://www.google.com.fj/)',
                'http://www.google.fi/'         =>
                    __('Finland', 'iptrank') . ' (http://www.google.fi/)',
                'http://www.google.fr/'         =>
                    __('France', 'iptrank') . ' (http://www.google.fr/)',
                'http://www.google.gm/'         =>
                    __('The Gambia', 'iptrank') . ' (http://www.google.gm/)',
                'http://www.google.ge/'         =>
                    __('Georgia', 'iptrank') . ' (http://www.google.ge/)',
                'http://www.google.de/'         =>
                    __('Germany', 'iptrank') . ' (http://www.google.de/)',
                'http://www.google.com.gi/'     =>
                    __('Gibraltar', 'iptrank') . ' (http://www.google.com.gi/)',
                'http://www.google.com.gr/'     =>
                    __('Greece', 'iptrank') . ' (http://www.google.com.gr/)',
                'http://www.google.gl/'         =>
                    __('Greenland', 'iptrank') . ' (http://www.google.gl/)',
                'http://www.google.gg/'         =>
                    __('Guernsey', 'iptrank') . ' (http://www.google.gg/)',
                'http://www.google.hn/'         =>
                    __('Honduras', 'iptrank') . ' (http://www.google.hn/)',
                'http://www.google.com.hk/'     =>
                    __('Hong Kong', 'iptrank') . ' (http://www.google.com.hk/)',
                'http://www.google.co.hu/'      =>
                    __('Hungary', 'iptrank') . ' (http://www.google.co.hu/)',
                'http://www.google.co.in/'      =>
                    __('India', 'iptrank') . ' (http://www.google.co.in/)',
                'http://www.google.co.id/'      =>
                    __('Indonesia', 'iptrank') . ' (http://www.google.co.id/)',
                'http://www.google.ie/'         =>
                    __('Ireland', 'iptrank') . ' (http://www.google.ie/)',
                'http://www.google.co.im/'      =>
                    __('Isle of Man', 'iptrank') . ' (http://www.google.co.im/)',
                'http://www.google.co.il/'      =>
                    __('Israel', 'iptrank') . ' (http://www.google.co.il/)',
                'http://www.google.it/'         =>
                    __('Italy', 'iptrank') . ' (http://www.google.it/)',
                'http://www.google.com.jm/'     =>
                    __('Jamaica', 'iptrank') . ' (http://www.google.com.jm/)',
                'http://www.google.co.jp/'      =>
                    __('Japan', 'iptrank') . ' (http://www.google.co.jp/)',
                'http://www.google.co.je/'      =>
                    __('Jersey', 'iptrank') . ' (http://www.google.co.je/)',
                'http://www.google.kz/'         =>
                    __('Kazakhstan', 'iptrank') . ' (http://www.google.kz/)',
                'http://www.google.co.kr/'      =>
                    __('Korea', 'iptrank') . ' (http://www.google.co.kr/)',
                'http://www.google.lv/'         =>
                    __('Latvia', 'iptrank') . ' (http://www.google.lv/)',
                'http://www.google.co.ls/'      =>
                    __('Lesotho', 'iptrank') . ' (http://www.google.co.ls/)',
                'http://www.google.li/'         =>
                    __('Liechtenstein', 'iptrank') . ' (http://www.google.li/)',
                'http://www.google.lt/'         =>
                    __('Lithuania', 'iptrank') . ' (http://www.google.lt/)',
                'http://www.google.lu/'         =>
                    __('Luxembourg', 'iptrank') . ' (http://www.google.lu/)',
                'http://www.google.mw/'         =>
                    __('Malawi', 'iptrank') . ' (http://www.google.mw/)',
                'http://www.google.com.my/'     =>
                    __('Malaysia', 'iptrank') . ' (http://www.google.com.my/)',
                'http://www.google.com.mt/'     =>
                    __('Malta', 'iptrank') . ' (http://www.google.com.mt/)',
                'http://www.google.mu/'         =>
                    __('Mauritius', 'iptrank') . ' (http://www.google.mu/)',
                'http://www.google.com.mx/'     =>
                    __('México', 'iptrank') . ' (http://www.google.com.mx/)',
                'http://www.google.ms/'         =>
                    __('Montserrat', 'iptrank') . ' (http://www.google.ms/)',
                'http://www.google.com.na/'     =>
                    __('Namibia', 'iptrank') . ' (http://www.google.com.na/)',
                'http://www.google.com.np/'     =>
                    __('Nepal', 'iptrank') . ' (http://www.google.com.np/)',
                'http://www.google.nl/'         =>
                    __('Netherlands', 'iptrank') . ' (http://www.google.nl/)',
                'http://www.google.co.nz/'      =>
                    __('New Zealand', 'iptrank') . ' (http://www.google.co.nz/)',
                'http://www.google.com.ni/'     =>
                    __('Nicaragua', 'iptrank') . ' (http://www.google.com.ni/)',
                'http://www.google.com.nf/'     =>
                    __('Norfolk Island', 'iptrank') . ' (http://www.google.com.nf/)',
                'http://www.google.com.pk/'     =>
                    __('Pakistan', 'iptrank') . ' (http://www.google.com.pk/)',
                'http://www.google.com.pa/'     =>
                    __('Panamá', 'iptrank') . ' (http://www.google.com.pa/)',
                'http://www.google.com.py/'     =>
                    __('Paraguay', 'iptrank') . ' (http://www.google.com.py/)',
                'http://www.google.com.pe/'     =>
                    __('Perú', 'iptrank') . ' (http://www.google.com.pe/)',
                'http://www.google.com.ph/'     =>
                    __('Philippines', 'iptrank') . ' (http://www.google.com.ph/)',
                'http://www.google.pn/'         =>
                    __('Pitcairn Islands', 'iptrank') . ' (http://www.google.pn/)',
                'http://www.google.pl/'         =>
                    __('Poland', 'iptrank') . ' (http://www.google.pl/)',
                'http://www.google.pt/'         =>
                    __('Portugal', 'iptrank') . ' (http://www.google.pt/)',
                'http://www.google.com.pr/'     =>
                    __('Puerto Rico', 'iptrank') . ' (http://www.google.com.pr/)',
                'http://www.google.cg/'         =>
                    __('Rep. of the Congo', 'iptrank') . ' (http://www.google.cg/)',
                'http://www.google.ro/'         =>
                    __('Romania', 'iptrank') . ' (http://www.google.ro/)',
                'http://www.google.ru/'         =>
                    __('Russia', 'iptrank') . ' (http://www.google.ru/)',
                'http://www.google.rw/'         =>
                    __('Rwanda', 'iptrank') . ' (http://www.google.rw/)',
                'http://www.google.sh/'         =>
                    __('Saint Helena', 'iptrank') . ' (http://www.google.sh/)',
                'http://www.google.sm/'         =>
                    __('San Marino', 'iptrank') . ' (http://www.google.sm/)',
                'http://www.google.com.sg/'     =>
                    __('Singapore', 'iptrank') . ' (http://www.google.com.sg/)',
                'http://www.google.sk/'         =>
                    __('Slovakia', 'iptrank') . ' (http://www.google.sk/)',
                'http://www.google.co.za/'      =>
                    __('South Africa', 'iptrank') . ' (http://www.google.co.za/)',
                'http://www.google.es/'         =>
                    __('Spain', 'iptrank') . ' (http://www.google.es/)',
                'http://www.google.se/'         =>
                    __('Sweden', 'iptrank') . ' (http://www.google.se/)',
                'http://www.google.ch/'         =>
                    __('Switzerland', 'iptrank') . ' (http://www.google.ch/)',
                'http://www.google.com.tw/'     =>
                    __('Taiwan', 'iptrank') . ' (http://www.google.com.tw/)',
                'http://www.google.co.th/'      =>
                    __('Thailand', 'iptrank') . ' (http://www.google.co.th/)',
                'http://www.google.tt/'         =>
                    __('Trinidad and Tobago', 'iptrank') . ' (http://www.google.tt/)',
                'http://www.google.com.tr/'     =>
                    __('Turkey', 'iptrank') . ' (http://www.google.com.tr/)',
                'http://www.google.com.ua/'     =>
                    __('Ukraine', 'iptrank') . ' (http://www.google.com.ua/)',
                'http://www.google.ae/'         =>
                    __('United Arab Emirates', 'iptrank') . ' (http://www.google.ae/)',
                'http://www.google.co.uk/'      =>
                    __('United Kingdom', 'iptrank') . ' (http://www.google.co.uk/)',
                'http://www.google.com.uy/'     =>
                    __('Uruguay', 'iptrank') . ' (http://www.google.com.uy/)',
                'http://www.google.uz/'         =>
                    __('Uzbekistan', 'iptrank') . ' (http://www.google.uz/)',
                'http://www.google.vu/'         =>
                    __('Vanuatu', 'iptrank') . ' (http://www.google.vu/)',
                'http://www.google.co.ve/'      =>
                    __('Venezuela', 'iptrank') . ' (http://www.google.co.ve/)',
            );
            return $urls;
        }
    }

}
?>